### This script creates input files for migrate using phased nexus data,
### a file that assigns locality to individuals,
### and a list of variable loci
### Written by Brad Nelson

import os, sys, re
locality_file = "./LocalityInfoSimple.txt"
varloc_file = "./variable.txt"
nexus_dir = "/Users/mike/Desktop/DATA!/SalixNexusPHASED/"
line_skip = 8
outgroup = {'Salbaxxxxx_a': '', 'Salbaxxxxx_b': ''}

def loc_list(locality_file, outgroup):       # Function that takes locality file and outgroup,
    locality_path = open(locality_file, 'r') # returns dictionary with locality and individual names
    loc = []                                 # as well as dictionary with population names
    locality = outgroup
    populations = {}

    for line in locality_path:    # Read each line in the locality file
        loc.append(line.split())  # Add locality and individual to list
        pop = line.split()[1]     
        populations[pop] = 0      # Add locality to dictionary
    locality_path.close()
    
    for line in loc:              # Cycle through lines in list
        individual = line[0]
        pop = line[1]
		# Make version of each name with 10 and 12 characters, then add them to locality
        numx = 10 - len(individual)
        ind_a = individual + 'x' * numx + '_a'
        ind_b = individual + 'x' * numx + '_b'
        ind8_a = individual + 'x' * (numx - 2) + '_a'
        ind8_b = individual + 'x' * (numx - 2) + '_b'
        locality[ind_a] = [line[1], ind8_a]
        locality[ind_b] = [line[1], ind8_b]
    
    return locality, populations
    
locality, populations = loc_list(locality_file, outgroup)

def variable_loci(varloc_file, nexus_dir): # Input list of locus names and directory of nexus files
    varloc = {}                            # Return dictionary with locus names paired with filename
    num_loci = 0                           # Also return total number of loci
    varloc_path = open(varloc_file, 'r')

    for line in varloc_path:
        locus = line.split()[0]
        varloc[locus] = ''
    varloc_path.close()

    dirList = os.listdir(nexus_dir)             # List all files in input directory

    for fname in dirList:
        locus = fname.split('.nex')[0]
        if locus in varloc:
            varloc[locus] = fname
        else:
            print locus + " mia"
    del(varloc['velvet_521'])                   # These loci would cause fatal errors in migrate
    del(varloc['velvet_155.231'])               # so I excluded them from the analysis
    del(varloc['Poptri_10-9487699..9489699'])
    del(varloc['Poptri_13-6896345..6898345'])
    del(varloc['Poptri_3-4426016..4428016'])
    del(varloc['Poptri_54-52394..54394'])
    del(varloc['Poptri_9-8323326..8325326'])
    del(varloc['Poptri_9-953407..955407'])
    del(varloc['velvet_157.124'])
    del(varloc['velvet_416'])
    for key in varloc:
        if varloc[key] != '':
            num_loci += 1
    return varloc, num_loci
    
varloc, num_loci = variable_loci(varloc_file, nexus_dir)
    
def pop_split(populations, varloc, nexus_dir, locality, line_skip): # Creates separate nexus-like file for each population
    pop = {}
    gene_string = {}                        # Will store names of loci
    pop_counter = populations
    locus_length = ''
    loc_counter = 0
    
    for x in populations.keys():            # Open a file for each population
        os.system('rm ./%s.txt' % x)
        pop[x] = open('./%s.txt' % x, 'a')
        gene_string[x] = ''                 # Add each population

    for key in sorted(varloc):              # Cycle through nexus files
        if varloc[key] == "":               # Ignore empty references
            pass
        else:
            loc_counter += 1
            print loc_counter, varloc[key]
            for a in pop_counter.keys():
                pop_counter[a] = 0
            reader = open(nexus_dir + varloc[key], 'r') # Open next nexus file
            counter = 0
            for line in reader:                         # Read lines
                counter += 1
                if counter < line_skip:
                    pass
                elif line[0:4] == 'end;':               # At file end, write number of sequences for each population
                    for location in populations.keys():
                        gene_string[location] = gene_string[location] + str(pop_counter[location]) + ' '
                elif line.count(';') == 0:              # If line is an indiv-sequence pair
                    indiv, seq = line.split()           # Store indiv and seq
                    locus_length_temp = len(seq)
                    if indiv in locality and locality[indiv] != '': # Check to make sure indiv is supposed to be there
                        location = locality[indiv][0]               # Store indiv's population name
                        pop[location].write(locality[indiv][1] + '  ' + seq + '\n') # Append string to pop file with indiv and gene sequence
                        pop_counter[location] += 1
                    elif indiv in locality and locality[indiv] == '':
                        pass
                    else:
                        print "Error: no match for %s" % indiv
                else:
                    pass
    
            locus_length = locus_length + str(locus_length_temp) + ' ' # Store locus lengths in string
            reader.close()
        
    for x in sorted(populations):
        gene_string[x] = gene_string[x] + ' ' + x
        pop[x].close()
        
    return gene_string, locus_length

gene_string, locus_length = pop_split(populations, varloc, nexus_dir, locality, line_skip)


def make_migrate_input(num_loci, locus_length, gene_string, populations, file_name, title): # Combine population files into migrate input file
    writer = open('./' + file_name, 'w')
    num_pop = len(populations)
    writer.write(str(num_pop) + ' ' + str(num_loci) + ' ' + title + '\n')
    writer.write(locus_length + '\n')
    
    for x in sorted(populations):
        reader = open('./' + x + '.txt', 'r')
        writer.write(gene_string[x] + '\n')
        for line in reader:
            writer.write(line)
            
        reader.close()
    writer.close()

file_name = 'migrate_input'
title = 'four_pop_Sal_m'
    
make_migrate_input(num_loci, locus_length, gene_string, populations, file_name, title)
    
    
